//------------------------------------------------------------------
//
//
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Gray
#property indicator_color2 Red
#property indicator_width2 2

#import "dynamicZone.dll"
   double dzSellP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i, double precision);
#import

extern int    Length           = 40;
extern int    MaPrice          = PRICE_CLOSE;
extern int    TemaLength       = 1;
extern int    PostLength       = 10;
extern int    PostTemaLength   = 1;
extern int    DzLookBackBars   = 50;
extern bool   arrowsVisible    = false;
extern string arrowsIdentifier = "MaAtrNmaArrows";
extern color  arrowsUpColor    = DeepSkyBlue;
extern color  arrowsDnColor    = Red;
extern double arrowsUpperGap   = 0.75;
extern double arrowsLowerGap   = 0.75;
extern bool   alertsOn         = false;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool alertsNotification = false;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;

double MA[];
double zeroLine[];
double trend[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,zeroLine);
   SetIndexBuffer(1,MA);
   SetIndexBuffer(2,trend);
   IndicatorShortName("dz nma atr ("+Length+")");
   return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--)
   {
      double close = iClose(NULL,0,i);
      double price = iMA(NULL,0,1,0,MODE_SMA,MaPrice,i);
      double ma    = iNma(price,iTema(price,TemaLength,i,0),Length,i,0);
      double atr   = iATR(NULL,0, Length, i);
      double value = 0;
      if (atr > 0.0) 
      {
         value = 100.0 * NormalizeDouble(MathAbs(close - ma)/atr,2);
            if (close < ma) value *= -1.0;
      }
      MA[i]=iNma(value,iTema(value,PostTemaLength,i,1),PostLength,i,1);
      zeroLine[i] = dzSellP(MA,0.5,DzLookBackBars, Bars, i, 0.001);
      trend[i] = trend[i+1];
         if (MA[i]>zeroLine[i]) trend[i] =  1;
         if (MA[i]<zeroLine[i]) trend[i] = -1;
         manageArrow(i);
   }
   manageAlerts();
   return(0);
}



//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

double workNma[][6];
#define _Prc 0
#define _Mom 1
#define _Nma 2

double iNma(double rawPrice, double price, int period, int i, int instanceNo=0)
{
   if (ArrayRange(workNma,0) != Bars) ArrayResize(workNma,Bars); i = Bars-i-1; instanceNo *= 3;
   
      workNma[i][instanceNo+_Prc] = price;
      workNma[i][instanceNo+_Mom] = workNma[i][instanceNo+_Prc]-workNma[i-1][instanceNo+_Prc];

      //
      //
      //
      //
      //
      
      double momRatio = 0.00;
      double sumMomen = 0.00;
      double ratio    = 0.00;
      
         for (int k = 0; k<period; k++)
         {
            sumMomen += MathAbs(workNma[i-k][instanceNo+_Mom]);
            momRatio +=         workNma[i-k][instanceNo+_Mom]*(MathSqrt(k+1)-MathSqrt(k));
         }
         if (sumMomen != 0) ratio = MathAbs(momRatio)/sumMomen;      
         workNma[i][instanceNo+_Nma] = workNma[i-1][instanceNo+_Nma]+ratio*(rawPrice-workNma[i-1][instanceNo+_Nma]);
   return(workNma[i][instanceNo+_Nma]);
}

//
//
//
//
//

double workTema[][6];
#define _ema1 0
#define _ema2 1
#define _ema3 2

double iTema(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workTema,0)!= Bars) ArrayResize(workTema,Bars); instanceNo*=3; r=Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workTema[r][_ema1+instanceNo] = workTema[r-1][_ema1+instanceNo]+alpha*(price                        -workTema[r-1][_ema1+instanceNo]);
          workTema[r][_ema2+instanceNo] = workTema[r-1][_ema2+instanceNo]+alpha*(workTema[r][_ema1+instanceNo]-workTema[r-1][_ema2+instanceNo]);
          workTema[r][_ema3+instanceNo] = workTema[r-1][_ema3+instanceNo]+alpha*(workTema[r][_ema2+instanceNo]-workTema[r-1][_ema3+instanceNo]);
   return(workTema[r][_ema3+instanceNo]+3.0*(workTema[r][_ema1+instanceNo]-workTema[r][_ema2+instanceNo]));
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible )
   {
         string lookFor = arrowsIdentifier+":"+Time[i]; ObjectDelete(lookFor);
         if (trend[i]!=trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,241,false);
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,242,true);
         }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap*arrowsUpperGap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap*arrowsLowerGap);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int forBar = 0;
      else      forBar = 1; 
      if (trend[forBar]!=trend[forBar+1])
      {
         if (trend[forBar]== 1) doAlert(0,"up");
         if (trend[forBar]==-1) doAlert(0,"down");
      }            
   }
} 
void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" dz ma atr - nma trend changed trend to "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotification) SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," dz ma atr - nma trend"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

